package cn.estsh.i3plus.core.apiservice.serviceimpl.busi;

import cn.estsh.i3plus.core.api.iservice.busi.ISysLocaleResourceService;
import cn.estsh.i3plus.platform.common.convert.ConvertBean;
import cn.estsh.i3plus.platform.common.tool.BeanCopyTool;
import cn.estsh.i3plus.platform.common.tool.SerializeTool;
import cn.estsh.i3plus.platform.common.util.CommonConstWords;
import cn.estsh.i3plus.platform.common.util.PlatformConstWords;
import cn.estsh.i3plus.pojo.base.bean.ListPager;
import cn.estsh.i3plus.pojo.base.common.Pager;
import cn.estsh.i3plus.pojo.base.common.PagerHelper;
import cn.estsh.i3plus.pojo.base.enumutil.CommonEnumUtil;
import cn.estsh.i3plus.pojo.base.util.StringUtil;
import cn.estsh.i3plus.pojo.platform.bean.SysLocaleLanguage;
import cn.estsh.i3plus.pojo.platform.bean.SysLocaleResource;
import cn.estsh.i3plus.pojo.platform.bean.SysMenu;
import cn.estsh.i3plus.pojo.platform.repository.SysLocaleLanguageRepository;
import cn.estsh.i3plus.pojo.platform.repository.SysLocaleResourceRepository;
import cn.estsh.i3plus.pojo.platform.sqlpack.CoreHqlPack;
import cn.estsh.impp.framework.boot.auth.AuthUtil;
import cn.estsh.impp.framework.boot.exception.ImppExceptionBuilder;
import cn.estsh.impp.framework.boot.exception.ImppExceptionEnum;
import cn.estsh.impp.framework.boot.util.ImppRedis;
import cn.estsh.impp.framework.boot.util.ValidatorBean;
import io.swagger.annotations.ApiOperation;
import org.apache.commons.lang3.StringUtils;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;

import javax.annotation.Resource;
import java.util.ArrayList;
import java.util.HashMap;
import java.util.List;
import java.util.Map;

/**
 * @Description : 系统资源
 * @Reference :
 * @Author : yunhao
 * @CreateDate : 2018-11-12 14:46
 * @Modify:
 **/
@Service
public class SysLocaleResourceService implements ISysLocaleResourceService {
    public static final Logger LOGGER = LoggerFactory.getLogger(SysLocaleLanguageService.class);

    @Autowired
    private SysLocaleResourceRepository sysLocaleResourceRDao;

    @Autowired
    private SysLocaleLanguageRepository sysLocaleLanguageRDao;

    @Resource(name= CommonConstWords.IMPP_REDIS_RES)
    private ImppRedis redisRes;


    @Override
    @ApiOperation(value = "新增系统资源")
    public SysLocaleResource insertSysLocaleResource(SysLocaleResource sysLocaleResource) {
        LOGGER.info("系统资源 SYS_LOCALE_RESOURCE SysLocaleResource：{}",sysLocaleResource);
        int count = sysLocaleResourceRDao.findByPropertyCount(new String[]{"languageCode","resourceKey"},
                        new String[]{sysLocaleResource.getLanguageCode(),sysLocaleResource.getResourceKey()});
        if (count > 0){
            throw ImppExceptionBuilder.newInstance()
                    .setSystemID(CommonEnumUtil.SOFT_TYPE.CORE.getCode())
                    .setErrorCode(ImppExceptionEnum.VARIFY_EXCEPTION_DATA_EXIT.getCode())
                    .setErrorDetail("语言资源已存在")
                    .setErrorSolution("请重新输入语言代码与资源键")
                    .build();
        }

        // 设置语言名称冗余
        SysLocaleLanguage sysLocaleLanguage = sysLocaleLanguageRDao.getByProperty("languageCode",
                                sysLocaleResource.getLanguageCode());
        ValidatorBean.checkNotNull(sysLocaleLanguage,"选中的语言不存在");
        sysLocaleResource.setLanguageNameRdd(sysLocaleLanguage.getLanguageName());

        return sysLocaleResourceRDao.insert(sysLocaleResource);
    }


    @Override
    public List<SysLocaleResource> saveSysLocaleResource(List<SysLocaleResource> list) {
        LOGGER.info("系统资源 SYS_LOCALE_RESOURCE list：{}",list);
        return sysLocaleResourceRDao.saveAll(list);
    }

    @Override
    @ApiOperation(value = "修改系统资源",notes = "修改系统资源")
    public void updateSysLocaleResource(SysLocaleResource sysLocaleResource) {
        SysLocaleResource originSysLocaleResource = sysLocaleResourceRDao.getById(sysLocaleResource.getId());
        LOGGER.info("系统资源 SYS_LOCALE_RESOURCE SysLocaleResource：{}", sysLocaleResource);
        String hqlPack = CoreHqlPack.packHqlSysLocaleResourceKey(sysLocaleResource);
        int count = sysLocaleResourceRDao.findByHqlWhereCount(hqlPack);
        if (count > 0) {
            throw ImppExceptionBuilder.newInstance()
                    .setSystemID(CommonEnumUtil.SOFT_TYPE.CORE.getCode())
                    .setErrorCode(ImppExceptionEnum.VARIFY_EXCEPTION_DATA_EXIT.getCode())
                    .setErrorDetail("语言资源已存在")
                    .setErrorSolution("请重新输入语言代码与资源键")
                    .build();
        }

        // 设置语言名称冗余
        SysLocaleLanguage sysLocaleLanguage = sysLocaleLanguageRDao.getByProperty("languageCode",
                sysLocaleResource.getLanguageCode());
        ValidatorBean.checkNotNull(sysLocaleLanguage, "选中的语言不存在");
        sysLocaleResource.setLanguageNameRdd(sysLocaleLanguage.getLanguageName());

        BeanCopyTool.beanCopy(sysLocaleResource, originSysLocaleResource);

        sysLocaleResourceRDao.update(originSysLocaleResource);
    }

    @Override
    @ApiOperation(value = "根据id删除系统资源",notes = "根据id删除系统资源")
    public void deleteSysLocaleResourceById(Long id) {
        LOGGER.info("系统资源 SYS_LOCALE_RESOURCE id：{}",id);
        sysLocaleResourceRDao.deleteById(id);
    }

    @Override
    @ApiOperation(value = "查询全部系统资源",notes = "查询全部系统资源")
    public List listSysLocaleResource() {
        LOGGER.info("系统资源 SYS_LOCALE_RESOURCE list");
        return sysLocaleResourceRDao.list();
    }

    @Override
    @ApiOperation(value = "系统资源复杂查询，分页，排序",notes = "系统资源复杂查询，分页，排序")
    public ListPager querySysLocaleResourceByPager(SysLocaleResource sysLocaleResource, Pager pager) {
        LOGGER.info("系统资源 SYS_LOCALE_RESOURCE SysLocaleResource：{}，Pager：{}",sysLocaleResource,pager);
        if(sysLocaleResourceRDao == null) {
            pager = PagerHelper.getPager(pager, sysLocaleResourceRDao.listCount());
            return new ListPager(sysLocaleResourceRDao.listPager(pager),pager);
        }else {
            String hqlPack = CoreHqlPack.packHqlSysLocaleResource(sysLocaleResource);
            pager = PagerHelper.getPager(pager, sysLocaleResourceRDao.findByHqlWhereCount(hqlPack));
            return new ListPager(sysLocaleResourceRDao.findByHqlWherePage(hqlPack + sysLocaleResource.orderBy(),pager),pager);
        }
    }

    @Override
    @ApiOperation(value = "根据id批量删除",notes = "根据id批量删除")
    public void deleteSysLocaleResourceByIds(Long[] ids) {
        LOGGER.info("系统资源 SYS_LOCALE_RESOURCE ids：{}",ids);
        sysLocaleResourceRDao.deleteByIds(ids);
    }

    @Override
    @ApiOperation(value = "批量添加")
    public void insertSysLocaleResourceList(List<SysLocaleResource> sysLocaleResourcesList) {
        // 添加校验
        SysLocaleResource sysLocaleResource = sysLocaleResourcesList.get(0);
        LOGGER.info("系统资源 SYS_LOCALE_RESOURCE SysLocaleResource：{}",sysLocaleResource);
        int count = sysLocaleResourceRDao.findByPropertyCount(new String[]{"languageCode","resourceKey"},
                new String[]{sysLocaleResource.getLanguageCode(),sysLocaleResource.getResourceKey()});
        if (count > 0){
            throw ImppExceptionBuilder.newInstance()
                    .setSystemID(CommonEnumUtil.SOFT_TYPE.CORE.getCode())
                    .setErrorCode(ImppExceptionEnum.VARIFY_EXCEPTION_DATA_EXIT.getCode())
                    .setErrorDetail("语言资源已存在")
                    .setErrorSolution("请重新输入语言代码与资源键")
                    .build();
        }

        // 设置语言名称冗余
        SysLocaleLanguage sysLocaleLanguage = sysLocaleLanguageRDao.getByProperty("languageCode",
                sysLocaleResource.getLanguageCode());
        ValidatorBean.checkNotNull(sysLocaleLanguage,"选中的语言不存在");
        sysLocaleResource.setLanguageNameRdd(sysLocaleLanguage.getLanguageName());

        sysLocaleResourceRDao.saveAll(sysLocaleResourcesList);
    }

    @Override
    @ApiOperation(value = "根据语言code和资源key查询资源")
    public SysLocaleResource getSysLocaleResourceByLanguageCodeAndResKey(String languageCode,String resKey) {
        return sysLocaleResourceRDao.getByProperty(new String[]{"languageCode","resourceKey"},new String[]{languageCode,resKey});
    }


    @Override
    public List<SysLocaleResource> findSysLocaleResourceByResKey(String resKey) {
        return sysLocaleResourceRDao.findByProperty(new String[]{"resourceKey"},new String[]{resKey});
    }

    @Override
    public SysLocaleResource getSysLocaleResourceById(Long id) {
        return sysLocaleResourceRDao.getById(id);
    }

    @Override
    public List<SysMenu> tranSysMenu(List<SysMenu> menuList) {
        if(menuList != null && menuList.size() > 0){
            String languageCode = (String) AuthUtil.getSessionLanguage();
            List<SysLocaleLanguage> languageList = sysLocaleLanguageRDao.list();
            menuList = (List<SysMenu>) SerializeTool.copyObject(menuList);
            Map<String,String> resourceMap = new HashMap<>();
            List<SysLocaleResource> insertList = new ArrayList<>();
            SysLocaleResource insert = null;
            String redisMenuKey = null;
            String redisMenuParentKey = null;

            for (SysMenu menu : menuList) {
                if(menu != null){
                    redisMenuKey = CommonConstWords.REDIS_PREFIX_CACHE_LANGUAGE +":" + menu.getName();
                    redisMenuParentKey = CommonConstWords.REDIS_PREFIX_CACHE_LANGUAGE +":" + menu.getParentId();
                    Map menuResourceMap = redisRes.getHashMap(redisMenuKey);
                    Map menuParentSesourceMap = redisRes.getHashMap(redisMenuParentKey);

                    if(menuResourceMap == null || menuResourceMap.size() <= 0){
                        menuResourceMap = new HashMap();
                        if(languageList != null){
                            for (SysLocaleLanguage language : languageList) {
                                insert = new SysLocaleResource();
                                insert.setResourceType(CommonEnumUtil.SYS_LOCALE_RESOURCE_TYPE.MENU.getValue());
                                insert.setLanguageCode(language.getLanguageCode());
                                insert.setLanguageNameRdd(language.getLanguageName());
                                insert.setResourceKey(menu.getName());
                                insert.setResourceValue(language.getLanguageName() + ":" + menu.getName());
                                insert.setIsSystem(CommonEnumUtil.TRUE_OR_FALSE.TRUE.getValue());
                                ConvertBean.serviceModelInitialize(insert, PlatformConstWords.SYSTEM_INIT_USER);

                                insertList.add(insert);
                                menuResourceMap.put(language.getLanguageCode(),insert.getResourceValue());
                            }
                        }

                        /* 保存 Redis */
                        redisRes.putHashMap(redisMenuKey, menuResourceMap, 0);
                    }

                    /* 菜单名称 国际化 */
                    if(menuResourceMap != null && menuResourceMap.size() > 0){
                        String menuName = (String)menuResourceMap.get(languageCode);
                        menu.setName(StringUtils.isNotBlank(menuName) ? menuName : menu.getName());
                    }

                    /* 功能菜单 父节点国际化 */
                    if(menuParentSesourceMap != null && menuParentSesourceMap.size() > 0){
                        String parentNameRdd = (String)menuParentSesourceMap.get(languageCode);
                        menu.setParentNameRdd(StringUtils.isNotBlank(parentNameRdd) ? parentNameRdd : menu.getParentNameRdd());
                    }
                }
            }

            /* 保存资源信息 */
            if(insertList != null && insertList.size() > 0){
                sysLocaleResourceRDao.saveAll(insertList);
            }
        }
        return menuList;
    }
}
